#include <windows.h>
#include <math.h>
#ifndef M_PI
#define	M_PI  3.14159265358979323846
#endif
#include <stdio.h>
#include "resource.h"

#define NAXE 10
HWND hDlgCamera = NULL;				// hwnd de la bote de dialogue
static WNDPROC StdEditCamProc;		// Memo procedure standard du controle d'dition
static HWND hEdit[NAXE];

#define VALIDE	500				// IDC bouton virtuel valide
#define ANNULE	501				// IDC bouton virtuel annule


/* Procedure interceptant les entres au control-edit IDC_EDIT1 pour simuler les
   action valide/annule avec RETURN/ESCAPE */
LRESULT CALLBACK CamEditProc (HWND hWnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
{
	/* Si reoit un return ou un escape,  on simule l'appui sur 
       des boutons virtuels valide ou annule */
    if (iMsg == WM_CHAR)
	{
		if(wParam == VK_RETURN)
		{
			SendMessage (hDlgCamera, WM_COMMAND, MAKELONG(VALIDE,0), (LPARAM) hWnd);
			return 0;
		}
		if (wParam == VK_ESCAPE)
		{
			SendMessage (hDlgCamera, WM_COMMAND, MAKELONG(ANNULE,0), (LPARAM) hWnd); 
			return 0;
		}
	}

    return CallWindowProc (StdEditCamProc, hWnd, iMsg, wParam, lParam) ;
}

static int idc_edit[NAXE] = {ID2_EDIT1, ID2_EDIT2, ID2_EDIT3, ID2_EDIT4, ID2_EDIT5,
							ID2_EDIT6, ID2_EDIT7, ID2_EDIT8, ID2_EDIT9, ID2_EDIT10};

static int idc_static[NAXE] = {IDC_STATIC1, IDC_STATIC2, IDC_STATIC3, IDC_STATIC4, IDC_STATIC5,
							IDC_STATIC6, IDC_STATIC7, IDC_STATIC8, IDC_STATIC9, IDC_STATIC10};

static int idc_scroll[NAXE] = {IDC_SCROLLBAR1, IDC_SCROLLBAR2, IDC_SCROLLBAR3, IDC_SCROLLBAR4, IDC_SCROLLBAR5,
							IDC_SCROLLBAR6, IDC_SCROLLBAR7, IDC_SCROLLBAR8, IDC_SCROLLBAR9, IDC_SCROLLBAR10};

static double Valmin[NAXE] = {-180., -180., -180., -20., -20., -10., -20., -20., 1.,  0.1};
static double Valmax[NAXE] = { 180.,  180.,  180.,  20.,  20.,  10.,  20.,  20., 100.,  10.};
static double val_mem[NAXE];
extern double conf_cam[];
void UpdateRov(int fmodconf, int fmodvue, int fmodsit);

/* Call back traitant les messages de la bote de dialogue */
BOOL CALLBACK DialogCamera(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam) 
{
#define MAXSCROLL 1000
static HWND hScroll[NAXE];
static int ix_mem[NAXE];
int ix, numax;
#define MAXCHAR1 20
char text[MAXCHAR1+1]="";
int n, lwp;

    switch (message) 
	{ 
	case WM_INITDIALOG :
//#define MODALE_cad_BLOQUE
#ifdef MODALE_cad_BLOQUE
		hDlgCamera = hDlg;
#endif
		/* On sous-classe le control-edit*/
		for (n=0; n<NAXE; n++)
		{
			hEdit[n] = GetDlgItem (hDlg, idc_edit[n]);
			StdEditCamProc = (WNDPROC) SetWindowLong(hEdit[n], GWL_WNDPROC, (LONG) CamEditProc);
			/* On affiche les valeurs initiales  l'afficheur et  l'editeur */
			if (n < 3) val_mem[n] = conf_cam[n]*180./M_PI;
			else val_mem[n] = conf_cam[n];
			sprintf(text,"%5.2lf",val_mem[n]);
			SetDlgItemText(hDlg, idc_edit[n], text);
			SetDlgItemText(hDlg, idc_static[n], text);
			/* On initialise le curseur */
			hScroll[n] = GetDlgItem (hDlg, idc_scroll[n]);
			SetScrollRange (hScroll[n] , SB_CTL, 0, MAXSCROLL, FALSE) ;
			/* On positionne le curseur  la valeur initiale */
			if (n < 8)
				ix_mem[n] = (int) (MAXSCROLL*(val_mem[n] - Valmin[n])/(Valmax[n]-Valmin[n]));
			else
				ix_mem[n] = (int) (MAXSCROLL*(log(val_mem[n]/Valmin[n]))/(log(Valmax[n]/Valmin[n])));
			SetScrollPos (hScroll[n], SB_CTL, ix_mem[n], TRUE) ;
		}
		return TRUE ;

    case WM_COMMAND :
		numax = -1;
		for (n=0; n<NAXE; n++) if(hEdit[n] == (HWND) lParam) { numax = n; break;}
		lwp = LOWORD(wParam);
        switch (lwp) 
		{ 
		case VALIDE : /* Valide dans fenetre edit */
			if (numax < 0) break;
			/* lecture texte du controle edition */
			n = GetDlgItemText(hDlg, idc_edit[numax], text, MAXCHAR1);
			/* test des valeurs / aux limites */
			if (n)
			{
				n = sscanf(text,"%lf",&val_mem[numax]);
				if (n!=1) val_mem[numax] = 0.;
				if (val_mem[numax] < Valmin[numax])
				{
					val_mem[numax] = Valmin[numax];
					MessageBeep(0xFFFFFFFF);
				}
				if (val_mem[numax] > Valmax[numax])
				{
					val_mem[numax] = Valmax[numax];
					MessageBeep(0xFFFFFFFF);
				}
			}
			/* On va afficher la valeur et mettre le curseur en place */
			if (numax < 8)
				ix_mem[numax] = (int) (MAXSCROLL*(val_mem[numax] - Valmin[numax])/(Valmax[numax]-Valmin[numax]));
			else
				ix_mem[numax] = (int) (MAXSCROLL*(log(val_mem[numax]/Valmin[numax]))/(log(Valmax[numax]/Valmin[numax])));
		case ANNULE : /* annule dans fenetre edit, ou fin du valide */
			/* On affiche et on positionne le curseur avec les valeurs _mem */
			if (numax < 0) break;
			sprintf(text,"%5.2lf",val_mem[numax]);
			SetDlgItemText(hDlg, idc_edit[numax], text);
			SetDlgItemText(hDlg, idc_static[numax], text);
			SetScrollPos (hScroll[numax], SB_CTL, ix_mem[numax], TRUE) ;
			if (numax < 3) conf_cam[numax] = val_mem[numax]*M_PI/180.;
			else conf_cam[numax] = val_mem[numax];
			if (lwp == VALIDE) UpdateRov(0,1+numax,0);
			return TRUE;

		case IDCANCEL :
#ifdef MODALE_cad_BLOQUE
			EndDialog (hDlg, 0) ;
#else
			DestroyWindow(hDlg);
#endif
			hDlgCamera = NULL;
//		    PostQuitMessage (0) ; // On envoie le WM_QUIT- On TUE l'application
			return TRUE ;
        } /* Fin du switch (LOWORD(wParam)) dans WM_COMMAND */
		break ; /* Fin du WM_COMMAND */

	case WM_VSCROLL :
	case WM_HSCROLL :
		numax = -1;
		for (n=0; n<NAXE; n++) if(hScroll[n] == (HWND) lParam) { numax = n; break;}
		if (numax < 0) return FALSE ;
		ix = ix_mem[numax];
		switch (LOWORD(wParam))
		{
		case SB_PAGEDOWN : 
			ix += MAXSCROLL/20 ;
			break ;
		case SB_LINEDOWN :
			++ix ;
			break ;
		case SB_PAGEUP : 
			ix -= MAXSCROLL/20 ;
			break ;
		case SB_LINEUP :
			--ix ;
			break ;
		case SB_TOP :
			ix = 0 ;
			break ;
		case SB_BOTTOM :
			ix = MAXSCROLL ;
			break ;
		case SB_THUMBPOSITION :
		case SB_THUMBTRACK :
			ix = HIWORD (wParam) ;
			break ;
		default :
			return FALSE ;
		} /* Fin du switch (LOWORD(wParam)) dans WM_VSCROLL */

		ix = ix < MAXSCROLL ? ix : MAXSCROLL ;
		ix = ix > 0 ? ix : 0 ;

		if (ix != ix_mem[numax])
		{
				ix_mem[numax] = ix ;
				SetScrollPos (hScroll[numax], SB_CTL, ix, TRUE) ;
				if (numax < 8)
					val_mem[numax] = Valmin[numax] + (Valmax[numax]-Valmin[numax])*ix_mem[numax]/MAXSCROLL;
				else
					val_mem[numax] = Valmin[numax] *pow(Valmax[numax]/Valmin[numax],(double) ix_mem[numax]/MAXSCROLL);
				sprintf(text,"%5.2lf",val_mem[numax]);
				SetDlgItemText(hDlg, idc_edit[numax], text);
				SetDlgItemText(hDlg, idc_static[numax], text);
				if (numax < 3) conf_cam[numax] = val_mem[numax]*M_PI/180.;
				else conf_cam[numax] = val_mem[numax];
				UpdateRov(0,1+numax,0);
		}
		return TRUE ; /* Fin du WM_VSCROLL */
	} /* Fin du switch (message) */

    return FALSE; 
} 
